/*
 * Copyright (C) 2001-2011 OSS Nokalva.  All rights reserved.
 *
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*
 * This example demonstrates how you can replace the default library fprintf()
 * with that of your own by using ossSetUserPrint().  As a result all the
 * tracing output is printed by OSSTraceFunc() instead of by fprintf().
 */


#include "bcas.h"			/* compiler-generated header file */
#include "printasn.h"

int OSSTraceFunc(OssGlobal *world, const char *lpszFormat, ...);



int main(void)
{

    OssBuf encodedData;			/* length and address of encoded data */
    BBCard *myCardPtr = NULL;		/* address of decoded data      */
    int  pdu_num = 0;
    struct ossGlobal w, *world = &w;
    char *buf;				/* saves address of encoded data */
    int retcode;			/* return code */


    retcode = ossinit(world, bcas);
    if (retcode) {
	printf("Initialization error = %d\n", retcode);
	return 1;
    }

    ossSetUserPrint(world, stdout, OSSTraceFunc);

    ossSetEncodingFlags(world, DEBUGPDU);
    ossSetDecodingFlags(world, DEBUGPDU);

    ossPrint(world, "\tUnencoded PDU...\n\n");
    ossPrintPDU(world, BBCard_PDU, &myCard);

    ossPrint(world, "\nThe encoder's trace messages...\n\n");

    encodedData.value = NULL;
    encodedData.length = 0;

    if (ossEncode(world, BBCard_PDU, &myCard, &encodedData)) {
	ossPrint(world, "%s\n", ossGetErrMsg(world));
	return 1;
    }
    else {
	buf = (char *)encodedData.value;    /* Save the address of the buffer
					     * with the encoded data. */

	ossPrint(world, "\nThe ossPrintPER() trace messages...\n\n");

	if (ossPrintPER(world, &pdu_num, &encodedData, (void **)&myCardPtr, 0, NULL)) {
	    ossPrint(world, "%s\n", ossGetErrMsg(world));
	    retcode = 1;
	 }
	 else {
		/*
		 * Print out the player's batting average.
		 */
	    ossPrint(world, "\n%s of the %s has a batting average of %0.3f\n",
	       myCardPtr->name, myCardPtr->team, myCardPtr->batting_average);

	    ossPrint(world, "\n\tDecoded PDU...\n\n");
	    ossPrintPDU(world, pdu_num, myCardPtr);
	    ossFreePDU(world, pdu_num, myCardPtr);
	    retcode = 0;
	}
		/*
		 * Free the encoder's output buffer.
		 */
	ossFreeBuf(world, buf);

    }
	/*
	 * Call ossterm() to free all resources.
	 */
    ossterm(world);
    return retcode;
}


int OSSTraceFunc(OssGlobal *world, const char *lpszFormat, ...)
{

#define nBufLen 1024
    va_list args;
    char szBuf[nBufLen];
    int nRC;


    memset(szBuf, '\0', nBufLen);

    va_start(args, lpszFormat);
    nRC = vsprintf(szBuf, lpszFormat, args);
    va_end(args);

    fputs(szBuf, world->asn1out);
    fflush(world->asn1out);

    return nRC;
}

